from treform.collector.colBasic import *
from treform.collector.colNaverNews import *
from treform.collector.colChosun import *
from treform.collector.colDonga import *
from treform.collector.colJoongang import *
from treform.collector.colBigkinds import *
from treform.collector.colNaverBlog import *